use crate::library::track::Track;
use crate::library::user::User;
use mongod::reference_of;
use mongod::Model;
use mongod::Referencable;
use mongodb::bson::doc;
use rocket::get;
use rocket::post;
use rocket::serde::json::Json;
use serde_json::json;

use crate::library::playlist::Playlist;
use crate::library::playlist::Visibility;
use crate::route::FallibleApiResponse;

use super::api_error;
use super::to_api;
use super::ToAPI;

#[get("/playlists")]
pub async fn playlists_route(u: User) -> FallibleApiResponse {
    let mut playlists = vec![json!({"id": "recent", "name": "Recently Played"})];

    let own_playlists = Playlist::find(doc! { "owner": u.reference()}, None)
        .await
        .unwrap();

    for playlist in own_playlists {
        playlists.push(json!({
            "id": playlist._id,
            "name": playlist.title
        }));
    }

    Ok(json!(playlists))
}

#[get("/playlist/<id>")]
pub async fn playlist_route(id: &str, u: User) -> FallibleApiResponse {
    if id == "recents" {
        // todo : recently played
    }

    let playlist = Playlist::get(id)
        .await
        .ok_or_else(|| api_error("No playlist with that ID found"))?;

    if matches!(playlist.visibility, Visibility::Private)
        && u.username != playlist.owner.get::<User>().await.username
    {
        return Err(api_error("Forbidden"));
    }

    Ok(playlist.api().await)
}

#[get("/playlist/<id>/tracks")]
pub async fn playlist_tracks_route(id: &str, u: User) -> FallibleApiResponse {
    if id == "recents" {
        // todo : recently played
    }

    let playlist = Playlist::get(id)
        .await
        .ok_or_else(|| api_error("No playlist with that ID found"))?;

    if matches!(playlist.visibility, Visibility::Private)
        && u.username != playlist.owner.get::<User>().await.username
    {
        return Err(api_error("Forbidden"));
    }

    let mut tracks: Vec<Track> = vec![];

    for track in playlist.tracks {
        tracks.push(track.get().await);
    }

    Ok(json!(to_api(&tracks).await))
}

#[derive(rocket::serde::Deserialize)]
pub struct PlaylistData {
    pub title: String,
    pub visibility: Visibility,
    pub tracks: Vec<String>,
}

#[post("/playlist", data = "<playlist>")]
pub async fn playlist_add_route(playlist: Json<PlaylistData>, u: User) -> FallibleApiResponse {
    let playlist = Playlist::create(
        u,
        &playlist.title,
        playlist.visibility.clone(),
        &playlist.tracks,
    )
    .await
    .ok_or_else(|| api_error("Failed to create playlist"))?;
    playlist.insert().await.unwrap();

    Ok(json!({"created": playlist._id}))
}

#[post("/playlist/<id>", data = "<edit>")]
pub async fn playlist_edit_route(
    id: &str,
    edit: Json<PlaylistData>,
    u: User,
) -> FallibleApiResponse {
    let mut playlist = Playlist::get(id)
        .await
        .ok_or_else(|| api_error("No playlist with that ID found"))?;

    if playlist.owner.id() != u._id {
        return Err(api_error("Forbidden"));
    }

    let mut tracks_ref = vec![];

    for track in &edit.tracks {
        tracks_ref
            .push(reference_of!(Track, track).ok_or_else(|| api_error("Invalid tracks found"))?);
    }

    playlist
        .update(&json!({
            "title": edit.title,
            "visibility": edit.visibility,
            "tracks": tracks_ref
        }))
        .await
        .unwrap();

    Ok(json!({"edited": playlist._id}))
}
